import com.toshiba.mwcloud.gs.*;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.net.InetAddress;
import java.net.UnknownHostException;
class Record // 200 bytes
{
    @RowKey long key;
    long a,b,c,d,e;
    long f,g,h,i,j;
    long k,l,m,n,o;
    long p,q,r,s,t;
    long u,v,w,x; 

}

class Operation extends Thread
{
    Properties props;
    Properties gsprops;
    GridStore store;
    long numOps;
    String colName;
    int threadId;
    
    public long getNumOps() {
        return this.numOps;
    }
    
    Operation(Properties props, final Properties gsprops, final int n) throws GSException {
        this.props = props;
        this.gsprops = gsprops;
        this.store = GridStoreFactory.getInstance().getGridStore(gsprops);
        this.threadId = n;
	try {
		InetAddress ip = InetAddress.getLocalHost();
        	this.colName = "col_" + ip.getHostName() + "_" + n;
	} catch (Exception e) {
		this.colName = "col_null_" + n;
	}
    }
}

class Reader extends Operation
{
    int batchSize;
    Collection<Long, Record> col;
    
    Reader(Properties props, final Properties gsprops, final int n) throws GSException {
        super(props, gsprops, n);
        System.out.println("Reading " + this.colName);
        this.col = this.store.putCollection(this.colName, Record.class);
        this.batchSize = Integer.parseInt(props.getProperty("batchSize"));
    }
    
    public void run() {
        long counter = 0;
        long iterations = Long.parseLong(this.props.getProperty("iterations"));
        try {
            for (int n = 0; n < iterations; n++) { 
                RowKeyPredicate<Long> rkp = RowKeyPredicate.create(Long.class);
                Map<String, RowKeyPredicate<?>> map = new HashMap<String, RowKeyPredicate<?>>();
                rkp.setStart(counter);
                rkp.setFinish(counter + this.batchSize - 1);
                map.put(this.colName, rkp);
                counter += this.batchSize;

                Map<String, List<Row>> res =  this.store.multiGet(map);
                List<Row> list = res.get(this.colName);
      //          System.out.println(this.colName + " has " + list.size() + " records with key: " + list.get(0).getLong(0));
                this.numOps += list.size();
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

class Writer extends Operation
{
    int batchSize;
    Collection<Long, Record> col;
    
    Writer(Properties props, final Properties gsprops, final int n) throws GSException {
        super(props, gsprops, n);
        System.out.println("Writing " + this.colName);
        this.col = this.store.putCollection(this.colName, Record.class);
        this.batchSize = Integer.parseInt(props.getProperty("batchSize"));
    }
    
    public void run() {
        long n = 0;
        long iterations = Long.parseLong(this.props.getProperty("iterations"));
        try {
            for (int k = 0; k < iterations; k++) {
                ArrayList<Record> list = new ArrayList<Record>();
                for (int i = 0; i < this.batchSize; i++) {
                    Record e = new Record();
                    e.key = n++;
                    list.add(e);
                }
                this.col.put(list);
                this.numOps += this.batchSize;
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

class Monitor extends Thread
{
    ArrayList<Operation> op_threads;
    long last_op_count;
    Date last_date;
    
    Monitor(ArrayList<Operation> op_threads) {
        this.op_threads = op_threads;
        this.last_op_count = 0;
        this.last_date = new Date();
    }

    public void run() {
        long n = 0;
        try {
            while (true) {
                Thread.sleep(5000);
                Iterator<Operation> iterator = this.op_threads.iterator();
                while (iterator.hasNext()) {
                    n += iterator.next().getNumOps();
                }
                Date last_date = new Date();
                System.out.print("op_count: " + n + " last_count: " + this.last_op_count + " time delta: " + (last_date.getTime() - this.last_date.getTime()));
                System.out.println(" ops/sec: " + (n - this.last_op_count) / ((last_date.getTime() - this.last_date.getTime()) / 1000.0));
                this.last_date = last_date;
                this.last_op_count = n;
                n = 0;
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}


public class OneMilOps
{
    public static void main(String[] args) throws GSException {
        Properties props = new Properties();
        Properties gsprops = new Properties();
        String s;

        if (args.length > 0) {
            s = args[0];
            System.out.println("Using " + s);
        }
        else {
            s = "onemilops.props";
        }
        try {
            props.load(new FileInputStream(s));
        }
        catch (Exception ex2) {
            System.out.println("Could not load onemilops.props");
            return;
        }
        try {
            gsprops.load(new FileInputStream("gridstore.props"));
        }
        catch (Exception ex3) {
            System.out.println("Could not load gridstore.props");
            return;
        }


        long numThreads = Long.parseLong(props.getProperty("numThreads"));
        Date date = new Date();
        ArrayList<Operation> threads = new ArrayList<Operation>();
        String operation = props.getProperty("operation");
        for (int n = 0; n < numThreads; n++) {
            if (operation.equals("write")) {
                Writer thread = new Writer(props, gsprops, n);
                threads.add(thread);
                thread.start();
            }
            else if (operation.equals("read")) {
                Reader thread = new Reader(props, gsprops, n);
                threads.add(thread);
                thread.start();
            }
            else {
                System.exit(1);
            }
        }

        Monitor monitor = new Monitor(threads);
        monitor.start();

        long op_count = 0;
        for (Operation thread : threads) {
            try {
                thread.join();
                op_count  += thread.getNumOps();
            }
            catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        Date date2 = new Date();
        System.out.println("Performed " + op_count + " "+ operation +" operations in " + (date2.getTime() - date.getTime()) / 1000.0 + " seconds or " + op_count / ((date2.getTime() - date.getTime()) / 1000.0) + " ops/sec");
        System.exit(0);
    }
}
